/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/08/22.
//

#include "grid.hpp"
#include "fggl/entity/gridworld/zone.hpp"

namespace demo {

	GridScene::GridScene(fggl::App &app) : Game(app), m_grid(nullptr) {
	}

	void GridScene::activate() {
		Game::activate();
		fggl::debug::log(fggl::debug::Level::info, "GridScene::activate()");

		// create the grid world
		m_grid = std::make_unique<fggl::entity::grid::Area2D<255,255>>(m_tiles);
	}

	void GridScene::deactivate() {
		m_grid = nullptr;
	}


	constexpr float DRAW_SIZE = 64.0F;
	constexpr float DRAW_HALF = DRAW_SIZE / 2.0F;

	static void render_grid(fggl::gfx::Paint& paint, fggl::entity::grid::Area2D<255, 255>& grid) {
		for (int i=0; i <= 31; ++i) {
			for (int j=0; j <= 31; ++j) {
				auto& cell = grid.floorAt(i, j);

				float x = i * DRAW_SIZE;
				float y = j * DRAW_SIZE;
				auto colour = (i + j) % 2 == 0 ? fggl::gfx::colours::WHITE : fggl::gfx::colours::BLACK;

				fggl::gfx::Path2D tileGfx = fggl::gfx::make_rect({x,y}, {DRAW_HALF, DRAW_HALF}, colour);
				paint.fill(tileGfx);
			}
		}
	}

	void GridScene::render(fggl::gfx::Graphics &gfx) {
		Game::render(gfx);

		fggl::gfx::Paint paint;
		render_grid(paint, *m_grid);

		// draw test shapes to check grid alignment
		for (int sides = 3; sides <= 25; ++sides) {
			auto hexTest = fggl::gfx::make_shape(fggl::math::vec2{sides, 5} * DRAW_SIZE, DRAW_HALF, sides, {1.0F - (sides / 25.0F), 0.0f, sides / 25.0f});
			paint.fill(hexTest);
		}

		gfx.draw2D(paint);
	}

} // namespace demo
